library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use work.binIO_Pkg.all;


-- The next entity gets its input by reading pcm-values from file
entity bmp_rw is
end bmp_rw;

architecture bmp_rw_arch of bmp_rw is

	constant N_HEADER_c : integer :=  1078; -- header size, offset to picture
	constant N_PICT_c   : integer := 38400; -- picture is 240(h) x 160(w)
	signal header_s		: byteArray_Typ (0 to N_HEADER_c-1);
	signal picture_s	: byteArray_Typ (0 to N_PICT_c-1);
	signal picture2_s	: byteArray_Typ (0 to N_PICT_c-1);
	signal Clk_s		: std_logic := '0';
	signal do_invert_s  : std_logic := '0';

begin

	Clk_s <= not Clk_s after 5 ns;

DO_INVERT_Lbl:
   process(do_invert_s)			
   begin
      for i in 0 to (N_PICT_c-1) loop
			picture2_s(i) <= not picture_s(i);
		end loop;
   end process DO_INVERT_Lbl;

BMP_READ_Lbl:
	process(Clk_s)
		file     bmpin_f    : binFile_Typ open read_mode is "sony_penguin.bmp";
		variable header_v   : byteArray_Typ (0 to N_HEADER_c-1);
		variable picture_v  : byteArray_Typ (0 to N_PICT_c-1);
	begin
		if rising_edge(Clk_s) then
			if not endfile(bmpin_f) then
				fread(bmpin_f,header_v,0);
				fread(bmpin_f,picture_v,0);
				header_s    <= header_v;
				picture_s   <= picture_v;
				do_invert_s <= '1' after 1 ns;
			else
				report "EndOfInputFile reached ..." severity FAILURE;
			end if;
		end if;
	end process BMP_READ_Lbl;
	
BMP_WRITE_Lbl:
	process(Clk_s)
		file     bmpout_f : byteFile_Typ open write_mode is "blackTux.bmp";
		variable header_v   : byteArray_Typ (0 to N_HEADER_c-1);
		variable picture2_v : byteArray_Typ (0 to N_PICT_c-1);
	begin
		if falling_edge(Clk_s) then
			header_v   := header_s;
			picture2_v := picture2_s;
			fwrite(bmpout_f,header_v,0);
			fwrite(bmpout_f,picture2_v,0);
		end if;
	end process BMP_WRITE_Lbl;

end architecture bmp_rw_arch;
